@@ -67,6 +67,7 @@ YFrameClient::YFrameClient(YWindow *parent, YFrameWindow *frame, Window win,
     fTimedOut(false),
     fFixedTitle(false),
     fIconize(true),
+    fMetaIcon(false),
     fPinging(false),
     fPingTime(0),
     fPid(0),
@@ -656,6 +657,9 @@ void YFrameClient::handleUnmap(const XUnmapEvent &unmap) {
 }
 
 void YFrameClient::handleProperty(const XPropertyEvent &property) {
+    if (destroyed())
+        return;
+
     bool new_prop = (property.state != PropertyDelete);
 
     if (property.window != handle()) {
@@ -851,6 +855,13 @@ void YFrameClient::handleDestroyWindow(const XDestroyWindowEvent &destroyWindow)
         manager->unmanageClient(this);
 }
 
+void YFrameClient::handleDamageNotify(const XDamageNotifyEvent& damage) {
+    if (destroyed())
+        return;
+
+    manager->handleDamageNotify(damage);
+}
+
 #ifdef CONFIG_SHAPE
 void YFrameClient::handleShapeNotify(const XShapeEvent &shape) {
     if (shapes.supported) {
@@ -956,6 +967,9 @@ void YFrameClient::setNetWMAllowedActions(Atom *actions, int count) {
 }
 
 void YFrameClient::handleClientMessage(const XClientMessageEvent &message) {
+    if (destroyed())
+        return;
+
     if (message.message_type == _XA_WM_CHANGE_STATE) {
         const long state = message.data.l[0];
         YFrameWindow* frame = getFrame();
@@ -1554,8 +1568,12 @@ void YFrameClient::obtainIcon() {
                 {
                     largestOffset = d;
                     largestSize = w;
+                    largestIcon = null;
                 }
             }
+            if (w == 128 && h == 128) {
+                fMetaIcon = true;
+            }
         }
 
         // Create missing icons by scaling the largest icon.
@@ -1622,6 +1640,25 @@ void YFrameClient::obtainIcon() {
     }
 }
 
+ref<YImage> YFrameClient::getMetaIcon() {
+    ref<YImage> image;
+    long count;
+    long* elem;
+
+    if (getNetWMIcon(&count, &elem)) {
+        for (long *e = elem;
+             e + 2 < elem + count && e[0] > 0 && e[1] > 0;
+             e += 2 + e[0] * e[1]) {
+            long w = e[0], h = e[1], *d = e + 2;
+            if (w == 128 && h == 128 && d + w*h <= elem + count) {
+                image = YImage::createFromIconProperty(d, w, h);
+                break;
+            }
+        }
+    }
+    return image;
+}
+
 bool YFrameClient::getKwmIcon(long* count, Pixmap** pixmap) {
     *count = 0;
     *pixmap = None;
